# -*- coding: utf-8 -*-
import irc3
from docopt import Dict as DocOptDict
from irc3.plugins.command import command
from irc3.utils import IrcString
from psycopg2 import Error

from . import DatabasePlugin


@irc3.plugin
class Tell(DatabasePlugin):
    requires = ['irc3.plugins.command',
                'nxy.plugins.storage']

    def __init__(self, bot: irc3.IrcBot):
        super().__init__(bot)
        self.tell_queue = {}

        # Fetch tells from database
        self.cur.execute('''
        select
          from_nick, to_nick, message
        from
          tells
        ''')

        # Add tells to queue
        for res in self.cur.fetchall():
            nick = res['to_nick']

            # Create list in queue and add tell
            if nick not in self.tell_queue:
                self.tell_queue[nick] = []
            self.tell_queue[nick].append([res['from_nick'],
                                          res['message']])

    @command
    def tell(self, mask: IrcString, target: IrcString, args: DocOptDict):
        """Saves a message for nick to forward on activity

        %%tell <nick> <message>...
        """
        nick = args['<nick>'].lower()
        tell = [mask.nick.lower(), nick, ' '.join(args['<message>']).strip()]

        # Create list in queue and add tell
        if nick not in self.tell_queue:
            self.tell_queue[nick] = []
        self.tell_queue[nick].append(tell)

        try:
            # Insert tell into database
            self.cur.execute('''
            insert into
              tells (from_nick, to_nick, message) 
            values
              (%s, %s, %s)
            ''', tell)
            self.con.commit()
        except Error:
            # Rollback transaction on error
            self.con.rollback()

    @irc3.event(r'(?i)^:(?P<mask>.*) PRIVMSG .* :.*')
    def check(self, mask: str):
        """If activ nick has tells, forward and delete them."""
        nick = IrcString(mask).nick

        if nick in self.tell_queue:
            # Forward all tells for nick
            for tell in self.tell_queue[nick]:
                self.bot.privmsg(nick, '[Tell] Message from {nick}: {message}'
                                 .format(nick=tell[0], message=tell[1]))

            # Remove nick from queue
            del self.tell_queue[nick]

            try:
                # Remove tells from database
                self.cur.execute('''
                delete from
                  tells
                where
                  to_nick = %s
                ''', [nick])
                self.con.commit()
            except Error:
                # Rollback transaction on error
                self.con.rollback()
